// CUBEGS
// Simple demo showing how to use geometry shader instancing
// A cube is built from 12 triangles. The geometry shader creates six from each, one per instance.
// By Ingemar Ragnemalm

// gcc geom3d2.c -o geom3d2 ../common/*.c ../common/Linux/MicroGlut.c -I ../common -I ../common/Linux/ -DGL_GLEXT_PROTOTYPES -lGL -lm -lX11

#define MAIN
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "LittleOBJLoader.h"
#include "LoadTGA.h"
// uses framework Cocoa
// uses framework OpenGL

#define near 1.

Model *cube;
// Reference to shader program
GLuint program;
GLuint tex;

void init(void)
{
	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_TRUE);
	printError("GL inits");

	// Load and compile shader
	program = loadShadersG("minimal.vert", "minimal.frag", "split.gs");
	glUseProgram(program);
	printError("init shader");
	
	// Upload geometry to the GPU:
	cube = LoadModel("cube.obj");
	
	mat4 camMatrix = lookAt(0,0,-2, 0,0,0, 0,1,0);
	mat4 projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 300.0);
	glUniformMatrix4fv(glGetUniformLocation(program, "camMatrix"), 1, GL_TRUE, camMatrix.m); // A "look-at" function could be more practical
	glUniformMatrix4fv(glGetUniformLocation(program, "projMatrix"), 1, GL_TRUE, projectionMatrix.m);
	
	printError("init arrays");
}

GLfloat a = 0.0;

void display(void)
{
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	a += 0.01;
	mat4 rot, trans, total;

	rot = Ry(a/2);
	trans = T(a/100, 0, 0);
	trans = T(0, 0, 0);
	total = rot * trans;
	glUniformMatrix4fv(glGetUniformLocation(program, "mdlMatrix"), 1, GL_FALSE, total.m);
	
	DrawModel(cube, program, "inPosition", NULL, NULL);

	printError("display");
	
	glutSwapBuffers();
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
//	glutInitContextVersion(4, 1);
	glutCreateWindow ("gs instancing");
	glutDisplayFunc(display);
	init ();
	glutRepeatingTimer(20);
	glutMainLoop();
	exit(0);
}
